<?php

namespace Schaffen\ServiceOrder\ServiceOrder\Models;


use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Schaffen\Core\Company\Traits\HasBranch;
use Schaffen\Core\Form\Models\FormAssociation;
use Schaffen\Core\Form\Traits\HasForms;
use Schaffen\Core\Person\Models\Person;
use Schaffen\Core\Person\Traits\HasAddress;
use Schaffen\Core\Person\Traits\HasPerson;
use Schaffen\Core\Visitation\Models\Visitation;
use Schaffen\Core\Visitation\Traits\HasVisitation;

class ServiceOrder extends Model
{

   use SoftDeletes,
      HasPerson,
      HasForms,
      HasAddress,
      HasVisitation,
      HasBranch;

   const STATUS_FATURADA = -1;
   const STATUS_EM_EXECUCAO = -6;
   const SERVICE_ORDER = 'OS';
   const QUOTE = 'ORCAMENTO';
   const DELETED_AT = 'os_deleted_at';
   const CREATED_AT = "os_date_insert";
   const UPDATED_AT = "os_date_update";
   public $prefix = 'os';
   protected $connection = "dynamicdb";
   protected $table = "os.os";
   protected $primaryKey = "os_codigo";
   protected $fillable = [
      "os_numero",
      "os_fk_pes_codigo",
      "os_numero_contrato",
      "os_end_cep",
      "os_end_logradouro",
      "os_end_numero",
      "os_end_bairro",
      "os_end_complemento",
      "os_end_cidade",
      "os_end_uf",
      "os_recursos",
      "os_campos",
      "os_data_agendamento",
      "os_observacao",
      "os_fk_fil_codigo",
      "os_fk_con_codigo",
      "os_end_distancia_km",
      "os_valor_total_km",
      "os_valor_total_hora_deslocamento",
      "os_valor_total_pedagio",
      "os_fk_tse_codigo",
      "os_fk_sto_codigo",
      "os_fk_usu_codigo_vendedor",
      "os_discriminacao_servico",
      "os_data_validade",
      "os_deleted_at",
      "os_fk_pee_codigo",
      "os_tipo",
      "os_numero_tipo",
      "os_planejada",
   ];
   protected $casts = [
      "os_recursos" => "json",
      "os_campos" => "json",
   ];


   public function status()
   {
      return $this->belongsTo(ServiceOrderStatus::class, 'os_fk_sto_codigo');
   }

   public function client()
   {
      return $this->belongsTo(Person::class, 'os_fk_pes_codigo');
   }

   public function items()
   {
      return $this->hasMany(ServiceOrderItem::class, 'osps_fk_os_codigo');
   }

   public function form_association()
   {
      return $this->hasMany(FormAssociation::class, 'foa_formeavel_codigo','os_codigo')
         ->where('foa_formeavel_tipo', 'service-order');
   }

   public function resources()
   {
      return $this->hasMany(ServiceOrderResource::class, 'osr_fk_os_codigo');
   }


}
