<?php
/**
 * Created for Schaffen IT.
 * User: Thiago Traczykowski
 */

namespace Schaffen\ServiceOrder\ServiceOrder\Services;


use Illuminate\Support\Facades\DB;
use Schaffen\Core\Company\Repositories\BranchRepository;
use Schaffen\Core\Person\Repositories\PersonRepository;
use Schaffen\ServiceOrder\ServiceOrder\Repositories\ServiceOrderRepository;
use Schaffen\ServiceOrder\ServiceOrder\Repositories\ServiceOrderStatusRepository;

class CreateOrUpdateAServiceOrder
{

   /**
    * @var ServiceOrderRepository
    */
   private ServiceOrderRepository $service_order_repository;


   public function __construct(
      ServiceOrderRepository $service_order_repository
   )
   {
      $this->service_order_repository = $service_order_repository;
   }

   public function storeOrUpdate(array $input, $service_order_id = null, $relations = [])
   {

      return DB::transaction(function () use ($input, $service_order_id, $relations) {


         if (!$service_order_id) {
            //Inicializar parametros default
            $input['os_data_agendamento'] = $input['os_data_agendamento'] ?? null;
            $input['os_data_validade'] = $input['os_data_validade'] ?? null;
            $input['os_recursos'] = $input['os_recursos'] ?? [];
            $input['os_campos'] = $input['os_campos'] ?? [];
            $input['products'] = $input['products'] ?? [];
            $input['services'] = $input['services'] ?? [];
            $input['items'] = $input['items'] ?? ($input['products'] ?? []) + ($input['services'] ?? []);
            $input['executions'] = $input['executions'] ?? [];
            $input['forms'] = $input['forms'] ?? [];


            if (!($input['os_fk_sto_codigo'] ?? null)) {
               $service_order_status_repository = app()->make(ServiceOrderStatusRepository::class);
               $input['os_fk_sto_codigo'] = $service_order_status_repository->first(["sto_status_inicial" => 1])->sto_codigo;
            }
         }

         if ($service_order_id) {
            //Inicializar parametros default para exclusão
            $input['executions_del'] = $input['executions_del'] ?? [];
            $input['items_del'] = $input['items_del'] ?? ($input['products_del'] ?? []) + ($input['services_del'] ?? []);

            $this->clearAssociativesOnEdit($input, $service_order_id);
         }

         $input['items'] = $input['items'] ?? ($input['products'] ?? []) + ($input['services'] ?? []);


         if ($input['os_fk_pes_codigo'] ?? null) {
            $person_repository = app()->make(PersonRepository::class);
            $person_db = $person_repository->find($input['os_fk_pes_codigo']);
            $relations[] = [
               "name" => "person", "model" => $person_db
            ];
         }

         if ($input['os_fk_fil_codigo'] ?? null) {
            $branch_repository = app()->make(BranchRepository::class);
            $branch_db = $branch_repository->find($input['os_fk_fil_codigo']);
            $relations[] = [
               "name" => "branch", 'model' => $branch_db,
            ];
         }

         if ($input['os_fk_sto_codigo'] ?? null) {
            $service_order_status_repository = app()->make(ServiceOrderStatusRepository::class);
            $status_db = $service_order_status_repository->find($input['os_fk_sto_codigo']);
            $relations[] = [
               "name" => "status", 'model' => $status_db,
            ];
         }


         if ($input['os_recursos']) {
            $input['os_recursos'] = $this->treatResources($input['os_recursos']);
         }

         $service_order_db = $this->service_order_repository->storeOrUpdate($input, $service_order_id, $relations);
         $input['os_codigo'] = $service_order_db->os_codigo;

         if ($input['os_recursos'] ?? false) {
            $create_or_update_items = app()->make(CreateOrUpdateServiceOrderResource::class);
            $create_or_update_items->handleResources($input['os_recursos'], $service_order_db->os_codigo);
         }

         if ($input['items'] ?? false) {
            $create_or_update_items = app()->make(CreateOrUpdateServiceOrderItem::class);
            $create_or_update_items->handleItems($input['items'], $input['os_recursos'], $service_order_db->os_codigo);

         }
         if ($input['executions'] ?? false) {
            $create_or_update_executions = app()->make(CreateOrUpdateServiceOrderExecution::class);
            $create_or_update_executions->handleExecutions($input['executions'], $service_order_db->os_codigo);

         }

         if ($input['forms'] ?? false) {
            $create_or_update_forms = app()->make(CreateOrUpdateServiceOrderForm::class);
            $create_or_update_forms->associateForms($input['forms'], $service_order_db->os_codigo);
         }

         return $service_order_db;
      });
   }

   private function clearAssociativesOnEdit($input, $service_order_id)
   {
//      $service_order_to_clear = $this->service_order_repository->find($service_order_id, ['resources']);
//
//      if (!empty($service_order_to_clear->resources)) {
//         $service_order_resource_remover = app()->make(RemoveServiceOrderResources::class);
//         $service_order_resource_remover->removeResources($service_order_to_clear->resources);
//      }

      if (count($input['executions_del']) > 0) {
         $remove_executions = app()->make(RemoveServiceOrderExecution::class);
         $remove_executions->removeExecutions($input['executions_del']);
      }

      if (count($input['items_del']) > 0) {
         $remove_items = app()->make(RemoveServiceOrderItem::class);
         $remove_items->removeItems($input['items_del']);
      }

   }

   private function treatResources($resources)
   {
      $person_repository = app()->make(PersonRepository::class);
      foreach ($resources as $key => $service_order_resource) {

         $person = $service_order_resource['osr_fk_vendedor_codigo'] ?? $service_order_resource['pes_codigo'] ?? $service_order_resource ?? null;
         if ($person) {
            $person_db = $person_repository->setSelect(
               ['pes_codigo', 'pes_nome', 'pes_identificador']
            )->find($person);

            $resources[$key] = [
               "recurso" => [
                  "pes_codigo" => $person_db->pes_codigo,
                  "pes_nome" => $person_db->pes_nome,
                  "pes_identificador" => $person_db->pes_identificador
               ],
               "pes_codigo" => $person_db->pes_codigo,
               "osr_fk_vendedor_codigo" => $person_db->pes_codigo,
               "comissao" => $service_order_resource['comissao'] ?? 0,
               "osr_percentual_comissao_produto" => $service_order_resource['osr_percentual_comissao_produto'] ?? 0,
               "osr_percentual_comissao_servico" => $service_order_resource['osr_percentual_comissao_produto'] ?? 0,
            ];
         }
      }
      return $resources;

   }

}
