<?php
/**
 * Created for Schaffen IT.
 * User: Thiago Traczykowski
 */

namespace Schaffen\ServiceOrder\ServiceOrder\Services;


use AlreadyBilled;
use Schaffen\ServiceOrder\ServiceOrder\Repositories\ServiceOrderItemRepository;
use Schaffen\ServiceOrder\ServiceOrder\Repositories\ServiceOrderRepository;
use Schaffen\ServiceOrder\ServiceOrder\Traits\SetServiceOrderOnAssociate;

class CreateOrUpdateServiceOrderItem
{

    use SetServiceOrderOnAssociate;

    /**
     * @var ServiceOrderItemRepository
     */
    private $service_order_item_repository;
    /**
     * @var CreateOrUpdateServiceOrderItemResources
     */
    private $create_or_update_service_order_item_resources;
    /**
     * @var CreateOrUpdateServiceOrderItemTaxes
     */
    private $create_or_update_service_order_item_taxes;
    /**
     * @var ServiceOrderItemRepository
     */
    private $service_order_item_resource_repository;
    /**
     * @var ServiceOrderRepository
     */
    private ServiceOrderRepository $service_order_repository;


    public function __construct(
        ServiceOrderItemRepository $service_order_item_repository,
        ServiceOrderItemRepository $service_order_item_resource_repository,
        ServiceOrderRepository $service_order_repository,
        CreateOrUpdateServiceOrderItemResources $create_or_update_service_order_item_resources,
        CreateOrUpdateServiceOrderItemTaxes $create_or_update_service_order_item_taxes
    )
    {
        $this->service_order_item_repository = $service_order_item_repository;
        $this->create_or_update_service_order_item_taxes = $create_or_update_service_order_item_taxes;
        $this->service_order_item_resource_repository = $service_order_item_resource_repository;
        $this->create_or_update_service_order_item_resources = $create_or_update_service_order_item_resources;
        $this->service_order_repository = $service_order_repository;
    }

    public function handleItems(
        array $items, $resources, $service_order_id
    )
    {
        $items = is_array($items) ? $items : [$items];
        $return = [];
        foreach ($items as $item) {

            if (!empty($item['osps_fk_fat_codigo'] ?? [])) {
                throw new AlreadyBilled(
                    "O item {$item['osps_codigo']} encontra-se faturado e não pode ser editado (faturamento código {$item['osps_fk_fat_codigo']})."
                );
            }
            /*
             * Tem que ter
             *
             * $item['osps_fk_pve_codigo']
             * $item['osps_quantidade']
             * $item['osps_valor_unitario']
            */

            /*
             *Inicializa com valor default
             *
             */
            $item['osps_fk_os_codigo'] = $service_order_id;
            $item['osps_codigo'] ??= null;
            $item['osps_desconto'] ??= 0;
            $item['osps_desconto_porcentagem'] ??= 0;
            $item['osps_serial'] ??= null;
            $item['osps_recursos'] ??= [];
            $item['osps_fk_cfop_codigo'] ??= null;
            $item['osps_ref_cliente'] ??= null;
            $item['osps_ref_cliente'] ??= null;
            $item['osps_prazo_entegra'] ??= null;
            $item['osps_numero_sequencia'] ??= null;
            $item['impostos_produto_servico'] ??= null;


//TODO PRA QUE DELETAR SE PODE DAR UPDATE? -- revisar

//         if ($item['osps_codigo'] ?? null) {
//            $this->service_order_item_resource_repository->destroyByItem($item['osps_codigo']);
//         }

            if (empty($item['osps_recursos'])) {
                $item['osps_recursos'] = array(
                    'pes_codigo' => auth()->user()->usu_fk_pes_codigo,
                    'comissao_produto' => 0,
                    'comissao_servico' => 0,
                );
            }

            $return[] = $item_db = $this->service_order_item_repository->storeOrUpdate($item, $item['osps_codigo'],
                $this->getServiceOrderRelation($service_order_id)
            );

            $this->create_or_update_service_order_item_resources->handleResources($resources, $item['osps_recursos'], $item_db->osps_codigo);
//         TODO Impostos estão quebrados no Baseline, verificar necessidade de arrumar e converter
//         $this->create_or_update_service_order_item_taxes->handleTaxes($resources, $item['osps_recursos'], item['osps_codigo']);
//
        }
        return $return;
    }
}
