<?php
/**
 * Created for Schaffen IT.
 * User: Thiago Traczykowski
 */

namespace Schaffen\ServiceOrder\ServiceOrder\Services;


class DestroyServiceOrder
{
   public function destroy($id)
   {
      try {
         DB::beginTransaction();
         $os = Os::find($id);
         if (!$os) {
            throw new SCHException("A ordem de serviço código {$id} não foi localizada.");
         }
         $osItens = OsProdutosServicos::where("osps_fk_os_codigo", $os->os_codigo)->get();

         $OsRecursos = OsRecursos::where('osr_fk_os_codigo', '=', $id);
         $OsRecursos->delete();

         foreach ($osItens as $item) {
            //verifica se nenhum dos itens foi faturado
            if (!empty($item->osps_fk_fat_codigo)) {
               $produto = ProdutoVariedadeEmbalagem::getDadosEmbalagem($item->osps_fk_pve_codigo);
               throw new SCHException("Existe um item faturado para esta OS: Produto: {$produto[0]->nome_completo}, código da fatura: {$item->osps_fk_fat_codigo}.");
            }
            if (!empty($item->osps_fk_cc_codigo_cred)) {
               $ccCred = ContaCorrente::find($item->osps_fk_cc_codigo_cred);
               if ($ccCred->cc_data_cancelamento == null) {
                  $this->cancelaMovimentacao($item->osps_fk_cc_codigo_cred, true);
               }
            }
            if (!empty($item->osps_fk_cc_codigo_deb)) {
               $ccDeb = ContaCorrente::find($item->osps_fk_cc_codigo_deb);
               if ($ccDeb->cc_data_cancelamento == null) {
                  $this->cancelaMovimentacao($item->osps_fk_cc_codigo_deb, true);
               }

            }
         }

         $logOs = new OsLog([
            "osl_fk_os_codigo" => $os->os_codigo,
            "osl_fk_too_codigo" => TipoOcorrenciaOs::REMOVIDA,
            "osl_ocorrencia" => "Orçamento excluído.",
            "osl_codigo_operacao" => $os->os_codigo
         ]);
         $logOs->save();
         DB::commit();

         $os->execucoes()->delete();
         $os->delete();
         DB::commit();

         return Response::success("Os removida com sucesso.", []);
      } catch (SCHException $e) {
         DB::rollback();
         return Response::faillure($e->getMessage(), $e->getTrace());
      } catch (\Exception $e) {
         DB::rollback();
         return Response::error($e->getMessage(), $e);
      }
   }
}
