<?php
/**
 * Created for Schaffen IT.
 * User: Thiago Traczykowski
 */

namespace Schaffen\ServiceOrder\ServiceOrder\Services;


class MoveInventory
{
   private function gerenciaEstoqueOs($codigoOs, $filial)
   {
//verifica se o status da OS controla estoque ou não

      $os = Os::find($codigoOs);

      if (!$os) {
         throw new SCHException("OS código {$codigoOs} não encontrada.");
      }

      $statusOs = StatusOs::find($os->os_fk_sto_codigo);
      if (!$statusOs) {
         throw new SCHException("Status da OS código {$os->os_fk_sto_codigo} não encontrado.");
      }

      if ($statusOs->sto_controla_estoque != StatusOs::SIM) {
//caso este status não controle estoque, ele não realiza as operações
         return;
      }

//obtém o local do estoque padrão da empresa
      $localEstoqueFilial = LocalEstoque::where("loe_fk_fil_codigo", $filial)
         ->where("loe_padrao_balcao", StatusOs::SIM)->first();

      if (!$localEstoqueFilial) {
         $filial = Filial::find($filial);
         throw new SCHException("Não existe um estoque de venda balcão definido para a filial {$filial->fil_descricao}.");
      }

//verifica se existe um local de estoque para as ORDENS de serviço na filial informada
      $localEstoqueOS = LocalEstoque::where("loe_fk_fil_codigo", $filial)
         ->where("loe_tipo", LocalEstoque::LOCAL_ORDEM_SERVICO)->first();

      if (!$localEstoqueOS) {
         $localEstoqueOS = new LocalEstoque([
            "loe_descricao" => "Estoque ordem de serviço",
            "loe_status" => 1,
            "loe_fk_pes_codigo" => null,
            "loe_fk_fil_codigo" => $filial,
            "loe_tipo" => LocalEstoque::LOCAL_ORDEM_SERVICO,
            "loe_padrao_balcao" => 0
         ]);
         $localEstoqueOS->save();
      }
//obtém as informações do item do estoque
      $itensOs = OsProdutosServicos::where("osps_fk_os_codigo", $codigoOs)
         ->where("osps_servico", self::PRODUTO)->get();

      if (count($itensOs) == 0) {
         return;
      }

      foreach ($itensOs as $itemOS) {
//verifica se já possui lançamentos ou venda
         if (empty($itemOS->osps_fk_cc_codigo_venda)) {
            if (!empty($itemOS->osps_fk_cc_codigo_deb)) {
               //cancela os lançamentos para criar outros novos
               $this->cancelaMovimentacao($itemOS->osps_fk_cc_codigo_cred, true);
               $this->cancelaMovimentacao($itemOS->osps_fk_cc_codigo_deb, true);
            }

            //retira o item do estoque de origem da FILIAL
            $itemOS->osps_fk_cc_codigo_deb = $this->geraLancamentos(
               $itemOS->osps_quantidade, $itemOS->osps_fk_pve_codigo, $itemOS->osps_valor_unitario, $localEstoqueFilial->loe_codigo, self::SAIDA, Operacao::ORDEM_SERVICO, $codigoOs, 0, $itemOS->osps_serial
            );
            //gera o lançamento no estoque de destino da filial
            $itemOS->osps_fk_cc_codigo_cred = $this->geraLancamentos(
               $itemOS->osps_quantidade, $itemOS->osps_fk_pve_codigo, $itemOS->osps_valor_unitario, $localEstoqueOS->loe_codigo, self::ENTRADA, Operacao::ORDEM_SERVICO, $codigoOs, 0, $itemOS->osps_serial
            );
            //salva as informações da movimentação gerada
            $itemOS->save();
         }
      }
   }
}
